var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Injectable } from '@angular/core';
import { HttpErrorResponse } from '@angular/common/http';
import { Observable, throwError } from 'rxjs';
// tslint:disable-next-line:import-blacklist
import { MatDialog } from '@angular/material';
// tslint:disable-next-line:import-blacklist
import { BehaviorSubject } from 'rxjs';
import { catchError, switchMap, filter, take, finalize } from 'rxjs/operators';
import { AuthComponent } from '../uix/auth/components/auth.components';
import { AuthService } from '../uix/auth/services/auth.services';
import { LoadingService } from '../uix/loading/services/loading.services';
import { UserService } from '../services/users.services';
import { LoginService } from '../services/login.services';
import { LogoutService } from '../services/logout.services';
import { ConfirmDialogComponent } from '../uix/confirm-dialog/confirm-dialog.component';
// import { AddressConfirmDialogComponent } from '../uix/address-confirm-dialog/address-confirm-dialog.component';
var ErrorHttpInterceptor = /** @class */ (function (_super) {
    __extends(ErrorHttpInterceptor, _super);
    function ErrorHttpInterceptor(loginService, logoutService, dialog, authService, loadingService, userService) {
        var _this = _super.call(this, authService, loadingService, userService) || this;
        _this.loginService = loginService;
        _this.logoutService = logoutService;
        _this.dialog = dialog;
        _this.authService = authService;
        _this.loadingService = loadingService;
        _this.userService = userService;
        _this.isRefreshingToken = false;
        _this.tokenSubject = new BehaviorSubject('');
        return _this;
    }
    ErrorHttpInterceptor.prototype.intercept = function (request, next) {
        var _this = this;
        return next.handle(request)
            .pipe(catchError(function (error) {
            if (error instanceof HttpErrorResponse) {
                if (error.status === 401) {
                    return _this.handle401Error(error, request, next);
                }
                else if (error.status === 504) {
                    _this.openErr('Upss! Parece que hubo un error de conexión', false);
                }
                else {
                    return _this.handleError(error);
                }
            }
            else {
                alert('Fallo de conexión a Internet');
                _this.logoutUser();
                return Observable.throw(error);
            }
            return Observable.throw(error);
        }));
    };
    ErrorHttpInterceptor.prototype.handle401Error = function (error, request, next) {
        var _this = this;
        var errorBody = error.error;
        if (errorBody.error === 'invalid_token') {
            if (errorBody.error_description.includes('Invalid refresh token (expired)')) {
                console.log(':: handle401Error :: error_description', errorBody.error_description);
                return this.logoutUser();
            }
        }
        else {
            return this.logoutUser();
        }
        if (!this.isRefreshingToken) {
            this.isRefreshingToken = true;
            this.tokenSubject.next('');
            return this.loginService.refreshToken().pipe(switchMap(function (newAccessToken) {
                if (newAccessToken) {
                    _this.tokenSubject.next(newAccessToken);
                    return next.handle(_this.addNewAccessToken(request, newAccessToken));
                }
                // If we don't get a new token, we are in trouble so logout.
                return _this.logoutUser();
            }), catchError(function (err) {
                console.log(err);
                // If there is an exception calling 'refreshToken', bad news so logout.
                return _this.logoutUser();
            }), finalize(function () {
                return _this.isRefreshingToken = false;
            }));
        }
        else {
            return this.tokenSubject.pipe(filter(function (accessToken) { return accessToken != null; }), take(1), switchMap(function (accessToken) {
                return next.handle(_this.addNewAccessToken(request, accessToken));
            }));
        }
    };
    ErrorHttpInterceptor.prototype.handleError = function (error) {
        var errorBody = error.error;
        var errorMessage;
        if (errorBody instanceof ErrorEvent) {
            errorMessage = errorBody.message;
        }
        else {
            var serverError = void 0;
            if (errorBody.message !== undefined) {
                if (errorBody.message.includes('org.springframework.')
                    || errorBody.message.includes('org.hibernate.')
                    || errorBody.message.includes('pre:LoginFilter')
                    || errorBody.message.includes('java.net.')) {
                    // aca excepciones para mensajes de backend que deben ser reemplazados
                    // ej: filterSession.
                    errorMessage = 'Error interno de servidor';
                }
                else {
                    errorMessage = errorBody.message;
                }
                serverError = errorBody.message;
            }
            else if (errorBody.error_description !== undefined) {
                if (errorBody.error === 'invalid_token') {
                    errorMessage = 'Token de sesion expirado';
                }
                else {
                    errorMessage = errorBody.error_description;
                }
                serverError = errorBody.error + ', ' + errorBody.error_description;
            }
            else if (error.message !== undefined) {
                this.openErr('!Uy¡ No estás conectado a internet', true);
                // alert('Fallo de conexión a Internet');
                // errorMessage = error.message;
                // serverError = error.message;
            }
            else {
                this.openErr('Upss! Parece que hubo un error de conexión', false);
                // alert('Fallo de conexión a Internet');
                // errorMessage = error.statusText;
                // serverError = error.statusText;
            }
            // console.error("A server-side error occurred");
            // console.log("Backend returned code: " + error.status);
            // console.log("serverError: " + serverError);
            // console.log("errorMessage: " + errorMessage);
        }
        // return an observable with a user-facing error message
        return throwError(errorMessage);
    };
    ErrorHttpInterceptor.prototype.addNewAccessToken = function (request, accessToken) {
        return request.clone({
            setHeaders: {
                Authorization: "Bearer " + accessToken
            }
        });
    };
    ErrorHttpInterceptor.prototype.logoutUser = function () {
        // Route to the login page (implementation up to you)
        // console.log(':: logoutUser :: logout');
        this.logoutService.logout();
        // tslint:disable-next-line: deprecation
        return Observable.throw('');
    };
    ErrorHttpInterceptor.prototype.openErr = function (msj, action) {
        this.updateLoading(false);
        this.openConfirmDialog(msj, action);
    };
    ErrorHttpInterceptor.prototype.openConfirmDialog = function (msj, action) {
        var _this = this;
        this.dialog.closeAll();
        var dialogRef = this.dialog.open(ConfirmDialogComponent, {
            width: '600px',
            position: { top: '10%' },
            data: {
                title: msj,
                question: action
            },
            disableClose: true
        });
        dialogRef.afterClosed().subscribe(function (result) {
            if (action) {
                _this.logoutUser();
            }
            else {
                window.location.reload();
            }
        });
    };
    ErrorHttpInterceptor = __decorate([
        Injectable(),
        __metadata("design:paramtypes", [LoginService, LogoutService, MatDialog,
            AuthService, LoadingService, UserService])
    ], ErrorHttpInterceptor);
    return ErrorHttpInterceptor;
}(AuthComponent));
export { ErrorHttpInterceptor };
//# sourceMappingURL=http-error.interceptors.js.map